--------------------------------------------------------------------------------
-- Jupiter-80 Librarian
--
-- Copyright 2014 Kim Sand
--
-- SVD file handling code
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Includes
--------------------------------------------------------------------------------
require "utils"
require "svdcommon"


--------------------------------------------------------------------------------
-- Functions
--------------------------------------------------------------------------------

function prompt_open_svd()
	init_svd_variables()

	filenamepath = renoise.app():prompt_for_filename_to_read({"*.svd"}, "Roland Jupiter-80/50 SVD file to open")

	if filenamepath == "" then
		renoise.app():show_status("File opening cancelled.")
		return false
	else
		filename = splitfilenamepath(filenamepath)
		svd_text = load_svd_file(filenamepath)

		if svd_text == false then
			svd_text = nil
			renoise.app():show_status("File opening failed.")
			return false
		end
	end

	renoise.app():show_status("File opened successfully.")

	return true
end

function load_svd_file(filename)
	local text = ""
	local f_in = io.open(filename, "rb")

	if f_in == nil then
		renoise.app():show_status("Couldn't open SVD file: " .. filename .. ".")
		return false
	end

	f_in:seek("set")
	text = f_in:read("*a")
	f_in:close()

	if check_validity(filename, text) == false then
		return false
	end

	return text
end

function check_validity(filename, text)
	-- Check for valid SVD file, string "SVD"
	if bytecmp(text, 0x02, {0x53, 0x56, 0x44}) == false then
		renoise.app():show_error(filename .. " is not a valid SVD file.")
		return false
	end

	-- Check that the SVD file is valid for Jupiter-80, strings "REG" and "JPTR"
	if bytecmp(text, 0x10, {0x52, 0x45, 0x47}) == true then
		if bytecmp(text, 0x14, {0x4A, 0x50, 0x54, 0x52}) == true then
			file_format = File_format.Jupiter80
		elseif bytecmp(text, 0x14, {0x4A, 0x50, 0x35, 0x30}) == true then
			file_format = File_format.Jupiter50
		end
	end

	if file_format == File_format.Unknown then
		renoise.app():show_error(filename .. " is not a valid Jupiter-80/50 file.")
		return false
	end

	return true
end

function prompt_save_text_from_svd(extension, dialog_text, save_text)
	local filename = renoise.app():prompt_for_filename_to_write(extension,
		dialog_text)

	if filename == "" then
		renoise.app():show_status("File saving cancelled.")
		return false
	else
	  local f_out = io.open(filename, "wb")

		if f_out == nil then
			renoise.app():show_status("Couldn't open file for writing: " .. filename .. ".")
			return false
		end

		f_out:seek("set")
		f_out:write(save_text)
		f_out:flush()
		f_out:close()

		renoise.app():show_status("File saved successfully as: " .. filename .. ".")
	end
end

function prompt_save_overview_from_svd()
	prompt_save_text_from_svd(".txt", "Save list as", overview_text)
end

function prompt_save_independents_from_svd()
	prompt_save_text_from_svd(".txt", "Save list as", independents_text)
end

function prompt_save_dependencies_from_svd()
	prompt_save_text_from_svd(".csv", "Save list as", dependencies_text)
end

function prompt_save_inverse_dependencies_from_svd()
	prompt_save_text_from_svd(".csv", "Save list as", inv_dependencies_text)
end

function prompt_save_modified_svd()
	prompt_save_text_from_svd(".svd", "Save modified SVD file as", svd_text)
end
