--------------------------------------------------------------------------------
-- Jupiter-80 Librarian
--
-- Copyright 2014 Kim Sand
--
-- SVD GUI functions
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Includes
--------------------------------------------------------------------------------
require "svdcommon"
require "svdfile"
require "svdutils"


--------------------------------------------------------------------------------
-- Functions
--------------------------------------------------------------------------------

function build_and_show_overview()
	local ok = true

	if svd_text == nil then
		ok = prompt_open_svd()
	end

	if ok and overview_text == nil then
		build_overview()
		renoise.app():show_status("Done building list of all sounds.")
	end

	if ok then
		show_overview(overview_text, build_and_show_overview, prompt_save_overview_from_svd)
	end
end

function build_and_show_independents()
	local ok = true

	if svd_text == nil then
		ok = prompt_open_svd()
	end

	if ok and overview_text == nil then
		build_overview()
	end

	if ok and dependencies_text == nil then
		build_dependencies()
	end

	if ok and independents_text == nil then
		build_independents()
		renoise.app():show_status("Done building list of independent sounds.")
	end

	if ok then
		show_overview(independents_text, build_and_show_independents, prompt_save_independents_from_svd)
	end
end

function build_and_show_dependencies()
	local ok = true

	if svd_text == nil then
		ok = prompt_open_svd()
	end

	if ok and overview_text == nil then
		build_overview()
	end

	if ok and dependencies_text == nil then
		build_dependencies()
		renoise.app():show_status("Done building dependencies.")
	end

	if ok then
		show_dependencies(dependencies_text, build_and_show_dependencies, prompt_save_dependencies_from_svd)
	end
end

function build_and_show_inverse_dependencies()
	local ok = true

	if svd_text == nil then
		ok = prompt_open_svd()
	end

	if ok and overview_text == nil then
		build_overview()
	end

	if ok and dependencies_text == nil then
		build_dependencies()
	end

	if ok and inv_dependencies_text == nil then
		build_inverse_dependencies()
		renoise.app():show_status("Done building inverse dependencies.")
	end

	if ok then
		show_dependencies(inv_dependencies_text, build_and_show_inverse_dependencies, prompt_save_inverse_dependencies_from_svd)
	end
end

function reorder_registrations()
	local ok = true

	if svd_text == nil then
		ok = prompt_open_svd()
	end

	if ok and overview_text == nil then
		build_overview()
	end

	if ok then
		show_reorder_matrix(reg_name_table, "registration", 8, 32, reorder_registrations, swap_registrations, false)

		renoise.app():show_status("Done building registration matrix.")
	end
end

function reorder_live_sets()
	local ok = true

	if svd_text == nil then
		ok = prompt_open_svd()
	end

	if ok and overview_text == nil then
		build_overview()
	end

	if ok and dependencies_text == nil then
		build_dependencies()
	end

	if ok then
		show_reorder_matrix(live_name_table, "live set", 10, 32, reorder_live_sets, swap_live_sets, false)

		renoise.app():show_status("Done building live set matrix.")
	end
end

function reorder_tones()
	local ok = true

	if svd_text == nil then
		ok = prompt_open_svd()
	end

	if ok and overview_text == nil then
		build_overview()
	end

	if ok and dependencies_text == nil then
		build_dependencies()
	end

	if ok then
		show_reorder_matrix(tone_name_table, "tone", 8, 32, reorder_tones, swap_tones, false)

		renoise.app():show_status("Done building tone matrix.")
	end
end

function show_overview(list_text, build_function, save_function)
	close_open_dialogs()

	local vb = renoise.ViewBuilder()

	local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
	local DIALOG_SPACING = renoise.ViewBuilder.DEFAULT_DIALOG_SPACING
	local CONTROL_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN

	-- Show the counts and names in a multiline text component
	local overview_column = vb:column {
		margin = CONTROL_MARGIN,
		vb:row {
			vb:button {
				text = "Open SVD file ...",
				notifier = function ()
					local ok = prompt_open_svd()

					if ok then
						build_function()
					end
				end
			},
			vb:button {
				text = "Save list to file ...",
				notifier = save_function
			}
		},
		vb:multiline_text {
			width = 400,
			height = 800,
			text = list_text
		}
	}

	local dialog_content = vb:column {
		  margin = DIALOG_MARGIN,
		  spacing = DIALOG_SPACING,
		  uniform = true,

		  overview_column
	}

	dialog_window = renoise.app():show_custom_dialog(
		filename,
		dialog_content
	)
end

function show_dependencies(dep_text, build_function, save_function)
	close_open_dialogs()

	local vb = renoise.ViewBuilder()

	local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
	local DIALOG_SPACING = renoise.ViewBuilder.DEFAULT_DIALOG_SPACING
	local CONTROL_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN

	-- Show the counts and names in a multiline text component
	local multiline_text = vb:column {
		margin = CONTROL_MARGIN,
		vb:row {
			vb:button {
				text = "Open SVD file ...",
				notifier = function ()
					local ok = prompt_open_svd()

					if ok then
						build_function()
					end
				end
			},
			vb:button {
				text = "Save list to file ...",
				notifier = save_function
			}
		},
		vb:multiline_text {
			width = 1000,
			height = 800,
			text = dep_text
		}
	}

	local dialog_content = vb:column {
		  margin = DIALOG_MARGIN,
		  spacing = DIALOG_SPACING,
		  uniform = true,

		  multiline_text
	}

	dialog_window = renoise.app():show_custom_dialog(
		filename,
		dialog_content
	)
end

function show_reorder_matrix(name_table, display_name, nr_of_columns, nr_of_rows, open_function, swap_function, keep_state)
	if keep_state == false then
		swap_from_part = nil
		swap_to_part = nil
		part_buttons = {}
		matrix_page = 1
	end

	close_open_dialogs()

	local vb = renoise.ViewBuilder()

	local CONTENT_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN
	local BUTTON_WIDTH = 120
	local BUTTON_DEFAULT_COLOR = {0, 0, 0}
	local BUTTON_SELECTED_COLOR = {255, 255, 255}

	local nr_of_pages = math.ceil(#name_table / (nr_of_rows * nr_of_columns))

	local dialog_content = vb:column {
		margin = CONTENT_MARGIN
	}

	local save_button = vb:button {
		text = "Save modified SVD file ...",
		notifier = prompt_save_modified_svd
	}

	local open_button = vb:button {
		text = "Open SVD file ...",
		notifier = function ()
			local ok = prompt_open_svd()

			if ok then
				open_function()
			end
		end
	}

	local button_row = vb:row {
		open_button,
		save_button
	}

	if nr_of_pages > 1 then
		for page = 1, nr_of_pages do
			local page_button = vb:button {
				text = "" .. page,
				notifier = function ()
					if matrix_page ~= page then
						matrix_page = page
						show_reorder_matrix(name_table, display_name, nr_of_columns, nr_of_rows, open_function, swap_function, true)
					end
				end
			}

			if matrix_page == page then
				page_button.color = BUTTON_SELECTED_COLOR
			end

			button_row:add_child(page_button)
		end
	end

	dialog_content:add_child(button_row)

	local loop_begin = (matrix_page - 1) * nr_of_rows + 1
	local loop_end = math.min(nr_of_rows, math.ceil(#name_table/nr_of_columns))

	if file_format == File_format.Jupiter50 then
		loop_end = math.min(128 / nr_of_columns, loop_end)
	end

	for bank = loop_begin, loop_begin - 1 + loop_end do
		local bank_row = vb:row {}

		for part_nr = ((bank - 1) * nr_of_columns) + 1, math.min(((bank - 1) * nr_of_columns) + nr_of_columns, #name_table) do
			local part_button = vb:button {
				width = BUTTON_WIDTH,
				text = name_table[part_nr],

				notifier = function()
					if swap_from_part == nil then
						swap_from_part = part_nr

						local from_button = part_buttons[swap_from_part]
						from_button.color = BUTTON_SELECTED_COLOR

						local from_name = from_button.text
						renoise.app():show_status("Selected " .. display_name ..  " \"" .. from_name .. "\" for swapping.")
					elseif swap_from_part == part_nr then
						local from_button = part_buttons[swap_from_part]
						from_button.color = BUTTON_DEFAULT_COLOR

						local from_name = from_button.text
						renoise.app():show_status("Deselected " .. display_name .. " \"" .. from_name .. "\" without swapping.")

						swap_from_part = nil
					else
						local from_button = part_buttons[swap_from_part]
						from_button.color = BUTTON_DEFAULT_COLOR
						swap_to_part = part_nr

						local to_button = part_buttons[swap_to_part]
						local from_name = from_button.text
						local to_name = to_button.text

						from_button.text = to_name
						to_button.text = from_name

						swap_function()

						renoise.app():show_status("Swapped " .. display_name .. "s \"" .. from_name .. "\" and \"" .. to_name .. "\".")
					end
				end
		  	}

			bank_row:add_child(part_button)
			part_buttons[part_nr] = part_button
		end

		dialog_content:add_child(bank_row)
	end

	dialog_window = renoise.app():show_custom_dialog(
		"Re-order " .. display_name .. "s",
		dialog_content
	)
end

function close_open_dialogs()
	if dialog_window and dialog_window.visible then
		dialog_window:close()
	end
end